import time
from contextlib import suppress

import rich_click as click
from rich import box
from rich.console import Group
from rich.panel import Panel
from rich.rule import Rule
from rich.status import Status
from rich.table import Table

from . import __pkg__, __version__
from ._api import Ahmia, console


@click.command()
@click.argument("query", type=str)
@click.option(
    "-e",
    "--export",
    is_flag=True,
    help="Export the output to a given filename",
)
@click.option(
    "-l",
    "--limit",
    default=20,
    show_default=True,
    help="Maximum number of results to show",
)
@click.option("--use-tor", is_flag=True, help="Connect to the Tor network")
def cli(query: str, limit: int, use_tor: bool, export: bool):
    """
    Search Ahmia for hidden services matching QUERY.
    """

    console.set_window_title(f"{__pkg__}, {__version__}")

    client = Ahmia(
        user_agent=f"{__pkg__}-cli/{__version__}; +https://pypi.org/project/{__pkg__}",
        use_tor=use_tor,
    )

    table = Table(
        box=box.SIMPLE,
        highlight=True,
        header_style="bold",
        border_style="dim",
    )
    table.add_column("#", style="bold")
    table.add_column("title")
    table.add_column("about")
    table.add_column("url", style="blue", no_wrap=True)
    table.add_column("last seen")

    now: float = time.time()
    try:

        with Status(
            "[bold]Checking for update[yellow]...[/bold][/yellow]", console=console
        ) as status:
            with suppress(Exception):
                client.check_updates()

            if use_tor:
                console.log(
                    "[bold][#c7ff70]✔ Routing traffic through Tor[/][/bold]",
                )
            else:
                console.log(
                    "[bold yellow]✘ Routing traffic through the clearnet[/bold yellow]"
                )
            status.update(
                f"[bold]Searching for [#c7ff70]{query}[/]. Please wait[yellow]...[/bold][/yellow]"
            )

            results, total_results = client.search(query=query, limit=limit)
            results_length = len(results)

            if total_results > 0:
                console.log(
                    f"[bold][#c7ff70]✔[/] Showing {results_length} of {total_results} results for [#c7ff70]{query}[/][/bold]"
                )
                for index, result in enumerate(results, start=1):
                    content_items = [
                        f"[bold][#c7ff70]{result.title}[/][/bold]",
                        Rule(style="#444444"),
                        result.about,
                        f"[blue][link=http://{result.url}]{result.url}[/link][/blue] — [bold]{result.last_seen_rel}[/]",
                    ]
                    console.print(
                        Panel(
                            Group(*content_items),
                            highlight=True,
                            border_style="dim",
                        )
                    )

                if export:
                    outfile: str = client.export_csv(results=results, path=query)
                    console.log(f"{results_length} results exported to {outfile}")

            else:
                console.log(
                    f"[bold][yellow]✘[/yellow]No results found for {query}.[/bold]"
                )

    except KeyboardInterrupt:
        console.log("\n[bold][red]✘[/red] User interruption detected[/bold]")

    except Exception as e:
        console.log(f"[bold][red]✘[/red] An error occurred:  [red]{e}[/red][/bold]")
    finally:
        elapsed: float = time.time() - now
        console.log(f"[bold][#c7ff70]✔[/] Finished in {elapsed:.2f} seconds.[/bold]")



import csv
import typing as t
from pathlib import Path
from types import SimpleNamespace

import requests
from bs4 import BeautifulSoup, ResultSet
from requests import Response
from requests_tor import RequestsTor
from rich.console import Console
from update_checker import UpdateChecker, UpdateResult

console = Console(log_time=False)


class Ahmia:

    def __init__(self, user_agent: str, use_tor: bool = False):
        self.user_agent = user_agent
        self.use_tor = use_tor

        if use_tor:
            self._search_url: str = (
                "http://juhanurmihxlp77nkq76byazcldy2hlmovfu2epvl5ankdibsot4csyd.onion/search/?q=%s"
            )
            self.session = RequestsTor(tor_ports=(9050,), tor_cport=(9051,))
        else:
            self._search_url: str = "https://ahmia.fi/search/?q=%s"
            self.session = requests.Session()

    @staticmethod
    def check_updates():
        from . import __pkg__, __version__

        checker = UpdateChecker()
        check: t.Union[UpdateResult, None] = checker.check(
            package_name=__pkg__, package_version=__version__
        )
        if check:
            console.print(check)

    @staticmethod
    def export_csv(results: t.Iterable[SimpleNamespace], path: str) -> str:
        results_list = list(results)

        if not all(isinstance(item, SimpleNamespace) for item in results_list):
            raise TypeError(
                "export_csv expects an iterable of SimpleNamespace objects (e.g., result of Ahmia.search())"
            )

        dict_rows = [item.__dict__ for item in results_list]

        if not dict_rows:
            raise ValueError("No results to export")

        out: Path = Path().home() / "pyahmia" / f"{path}.csv"
        out.parent.mkdir(parents=True, exist_ok=True)

        with out.open(mode="w", encoding="utf-8", newline="") as file:
            writer = csv.DictWriter(file, fieldnames=dict_rows[0].keys())
            writer.writeheader()
            writer.writerows(dict_rows)

        return str(out)

    def search(self, query: str, limit: int = 20) -> tuple[list[SimpleNamespace], int]:
        soup: BeautifulSoup = self._get_page_source(url=self._search_url % query)
        items: ResultSet = soup.find_all("li", {"class": "result"})
        total_count = len(items)

        results: list[SimpleNamespace] = []
        for item in items[:limit]:
            last_seen_tag = item.find("span", {"class": "lastSeen"})
            last_seen_text = (
                last_seen_tag.get_text(strip=True) if last_seen_tag else "NaN"
            )
            last_seen_timestamp = (
                last_seen_tag.get("data-timestamp") if last_seen_tag else "NaN"
            )

            results.append(
                SimpleNamespace(
                    **{
                        "title": " ".join(item.find("h4").text.split()),
                        "about": " ".join(item.find("p").text.split()),
                        "url": " ".join(item.find("cite").text.split()),
                        "last_seen_rel": last_seen_text.replace("\xa0", " "),
                        "last_seen_ts": last_seen_timestamp,
                    }
                )
            )

        return results, total_count

    def _get_page_source(self, url: str) -> BeautifulSoup:
        response: Response = self.session.get(
            url=url, headers={"User-Agent": self.user_agent}
        )
        soup: BeautifulSoup = BeautifulSoup(response.content, "html.parser")
        return soup
